@@ -1,5 +1,5 @@
 /* Enable debugging mode. */
-#undef DEBUG
+#undef DEBUG_MODE
 
 @BOTTOM@
 
