@@ -1,7 +1,7 @@
 /*
 hts.c
 
-Copyright (C) 1999 Lars Brinkhoff.  See COPYING for terms and conditions.
+Copyright (C) 1999, 2000 Lars Brinkhoff.  See COPYING for terms and conditions.
 
 hts is the server half of httptunnel.  httptunnel creates a virtual
 two-way data path tunneled in HTTP requests.
@@ -20,11 +20,14 @@ typedef struct
 {
   char *me;
   char *device;
+  const char *host;
   int port;
   char *forward_host;
   int forward_port;
   size_t content_length;
   char *pid_filename;
+  int use_std;
+  int use_daemon;
   int strict_content_length;
   int keep_alive;
   int max_connection_age;
@@ -40,7 +43,7 @@ usage (FILE *f, const char *me)
 "Usage: %s [OPTION]... [PORT]\n"
 "Listen for incoming httptunnel connections at PORT (default port is %d).\n"
 "When a connection is made, I/O is redirected to the destination specified\n"
-"by the --device or --forward-port switch.\n"
+"by the --device, --forward-port or --stdin-stdout switch.\n"
 "\n"
 "  -c, --content-length BYTES     use HTTP PUT requests of BYTES size\n"
 "                                 (k, M, and G postfixes recognized)\n"
@@ -58,8 +61,11 @@ usage (FILE *f, const char *me)
 #endif
 "  -M, --max-connection-age SEC   maximum time a connection will stay\n"
 "                                 open is SEC seconds (default is %d)\n"
+"  -s, --stdin-stdout             use stdin/stdout for communication\n"
+"                                 (implies --no-daemon)\n"
 "  -S, --strict-content-length    always write Content-Length bytes in requests\n"
 "  -V, --version                  output version information and exit\n"
+"  -w, --no-daemon                don't fork into the background\n"
 "  -p, --pid-file LOCATION        write a PID file to LOCATION\n"
 "\n"
 "Report bugs to %s.\n",
@@ -75,12 +81,15 @@ parse_arguments (int argc, char **argv, Arguments *arg)
   /* defaults */
 
   arg->me = argv[0];
+  arg->host = NULL;
   arg->port = DEFAULT_HOST_PORT;
   arg->device = NULL;
   arg->forward_host = NULL;
   arg->forward_port = -1;
   arg->content_length = DEFAULT_CONTENT_LENGTH;
   arg->pid_filename = NULL;
+  arg->use_std = FALSE;
+  arg->use_daemon = TRUE;
   arg->strict_content_length = FALSE;
   arg->keep_alive = DEFAULT_KEEP_ALIVE;
   arg->max_connection_age = DEFAULT_CONNECTION_MAX_TIME;
@@ -91,8 +100,10 @@ parse_arguments (int argc, char **argv, Arguments *arg)
       static struct option long_options[] =
       {
 	{ "help", no_argument, 0, 'h' },
-	{ "strict", no_argument, 0, 'S' },
+	{ "stdin-stdout", no_argument, 0, 's' },
+	{ "strict-content-length", no_argument, 0, 'S' },
 	{ "version", no_argument, 0, 'V' },
+	{ "no-daemon", no_argument, 0, 'w' },
 #ifdef DEBUG_MODE
 	{ "debug", optional_argument, 0, 'D' },
 	{ "logfile", required_argument, 0, 'l' },
@@ -106,7 +117,7 @@ parse_arguments (int argc, char **argv, Arguments *arg)
 	{ 0, 0, 0, 0 }
       };
 
-      static const char *short_options = "c:d:F:hk:M:p:SV"
+      static const char *short_options = "c:d:F:hk:M:p:sSVw"
 #ifdef DEBUG_MODE
 	"D:l:"
 #endif
@@ -120,10 +131,10 @@ parse_arguments (int argc, char **argv, Arguments *arg)
       switch (c)
 	{
 	case 0:
-	  printf ("option %s", long_options[option_index].name);
+	  fprintf (stderr, "option %s", long_options[option_index].name);
 	  if (optarg)
-	    printf (" with arg %s", optarg);
-	  printf ("\n");
+	    fprintf (stderr, " with arg %s", optarg);
+	  fprintf (stderr, "\n");
 	  break;
 	  
 	case 'c':
@@ -176,6 +187,11 @@ parse_arguments (int argc, char **argv, Arguments *arg)
 	  arg->max_connection_age = atoi (optarg);
 	  break;
 
+	case 's':
+	  arg->use_std=TRUE;
+	  arg->use_daemon=FALSE;
+	  break;
+
 	case 'S':
 	  arg->strict_content_length = TRUE;
 	  break;
@@ -188,34 +204,50 @@ parse_arguments (int argc, char **argv, Arguments *arg)
 	  arg->pid_filename = optarg;
 	  break;
 
+	case 'w':
+	  arg->use_daemon = FALSE;
+	  break;
+
 	case '?':
 	  break;
 
 	default:
-	  printf ("?? getopt returned character code 0%o ??\n", c);
+	  fprintf (stderr, "?? getopt returned character code 0%o ??\n", c);
 	}
     }
 
   if (argc - 1 == optind)
-    arg->port = atoi (argv[optind]);
+    {
+      char *colon = strchr (argv[optind], ':');
+
+      if (colon)
+	{
+	  *colon = '\0';
+	  arg->host = argv[optind];
+	  arg->port = atoi (colon + 1);
+	}
+      else
+	arg->port = atoi (argv[optind]);
+    }
   else if (argc - 1 > optind)
     {
       usage (stderr, arg->me);
       exit (1);
     }
 
-  if (arg->device == NULL && arg->forward_port == -1)
+  if (arg->device == NULL && arg->forward_port == -1 && !arg->use_std)
     {
-      fprintf (stderr, "%s: one of --device or --forward-port must be used.\n"
+      fprintf (stderr, "%s: one of --device, --forward-port or --stdin-stdout must be used.\n"
 	               "%s: try '%s -help' for help.\n",
 	       arg->me, arg->me, arg->me);
       exit (1);
     }
 
-  if (arg->device != NULL && arg->forward_port != -1)
+  if ((arg->device != NULL && arg->forward_port != -1) ||
+      (arg->device != NULL && arg->use_std) ||
+      (arg->forward_port != -1 && arg->use_std))
     {
-      fprintf (stderr, "%s: --device can't be used together with "
-	                   "--forward-port.\n"
+      fprintf (stderr, "%s: only one of --device, --forward-port or --stdin-stdout can be used.\n"
 	               "%s: try '%s --help' for help.\n",
 	       arg->me, arg->me, arg->me);
       exit (1);
@@ -228,8 +260,9 @@ parse_arguments (int argc, char **argv, Arguments *arg)
       exit (1);
     }
 
-  if (((arg->device == NULL) == (arg->forward_port == -1)) ||
-      arg->port == -1 ||
+  /* Removed test ((arg->device == NULL) == (arg->forward_port == -1))
+   * by Sampo Niskanen - those have been tested already! */
+  if (arg->port == -1 ||
       ((arg->forward_host == NULL) != (arg->forward_port == -1)))
     {
       usage (stderr, arg->me);
@@ -248,12 +281,12 @@ main (int argc, char **argv)
 
   parse_arguments (argc, argv, &arg);
 
-  if (debug_level == 0 || debug_file != NULL)
+  if ((debug_level == 0 || debug_file != NULL) && arg.use_daemon)
     daemon (0, 1);
 
 #ifdef DEBUG_MODE
   if (debug_level != 0 && debug_file == NULL)
-    debug_file = stdout;
+    debug_file = stderr;
 #else
   openlog ("hts", LOG_PID, LOG_DAEMON);
 #endif
@@ -261,16 +294,21 @@ main (int argc, char **argv)
   log_notice ("hts (%s) %s started with arguments:", PACKAGE, VERSION);
   log_notice ("  me = %s", arg.me);
   log_notice ("  device = %s", arg.device ? arg.device : "(null)");
-  log_notice ("  port = %d", arg.port);
+  if (arg.host)
+    log_notice ("  port = %s:%d", arg.host, arg.port);
+  else
+    log_notice ("  port = %d", arg.port);
   log_notice ("  forward_port = %d", arg.forward_port);
   log_notice ("  forward_host = %s",
 	      arg.forward_host ? arg.forward_host : "(null)");
   log_notice ("  content_length = %d", arg.content_length);
+  log_notice ("  strict_content_length = %d", arg.strict_content_length);
+  log_notice ("  use_std = %d", arg.use_std);
   log_notice ("  debug_level = %d", debug_level);
   log_notice ("  pid_filename = %s",
 	      arg.pid_filename ? arg.pid_filename : "(null)");
 
-  tunnel = tunnel_new_server (arg.port, arg.content_length);
+  tunnel = tunnel_new_server (arg.host, arg.port, arg.content_length);
   if (tunnel == NULL)
     {
       log_error ("couldn't create tunnel", argv[0]);
@@ -296,7 +334,7 @@ main (int argc, char **argv)
   signal (SIGPIPE, SIG_IGN);
 #endif
 
-  if(arg.pid_filename != NULL)
+  if (arg.pid_filename != NULL)
     {
       pid_file = fopen (arg.pid_filename, "w+");
       if (pid_file == NULL)
@@ -331,6 +369,26 @@ main (int argc, char **argv)
 			 arg.device, strerror (errno));
 	      log_exit (1);
 	    }
+	  /* Check that fd is not 0 (clash with --stdin-stdout) */
+	  if (fd == 0)
+	    {
+	      log_notice("changing fd from %d to 3",fd);
+	      if (dup2(fd,3) != 3)
+	        {
+		  log_error ("couldn't dup2(%d,3): %s",fd,strerror(errno));
+		  log_exit (1);
+		}
+	    }
+	} else if (arg.use_std) {
+	  log_debug ("using stdin as fd");
+	  fd = 0;
+	  if (fcntl(fd,F_SETFL,O_NONBLOCK)==-1)
+	    {
+	      log_error ("couldn't set stdin to non-blocking mode: %s",
+			 strerror(errno));
+	      log_exit (1);
+	    }
+	  /* Usage of stdout (fd = 1) is checked later. */
 	}
 
       if (tunnel_accept (tunnel) == -1)
@@ -338,7 +396,6 @@ main (int argc, char **argv)
 	  log_notice ("couldn't accept connection: %s", strerror (errno));
 	  continue;
 	}
-      log_notice ("connected to FIXME:hostname:port");
 
       if (arg.forward_port != -1)
 	{
@@ -360,6 +417,16 @@ main (int argc, char **argv)
 			 arg.forward_host, arg.forward_port, strerror (errno));
 	      log_exit (1);
 	    }
+	  /* Check that fd is not 0 (clash with --stdin-stdout) */
+	  if (fd == 0)
+	    {
+	      log_notice("changing fd from %d to 3",fd);
+	      if (dup2(fd,3) != 3)
+	        {
+		  log_error ("couldn't dup2(%d,3): %s",fd,strerror(errno));
+		  log_exit (1);
+		}
+	    }
 	}
 
       closed = FALSE;
@@ -409,7 +476,10 @@ main (int argc, char **argv)
 	}
 
       log_debug ("closing tunnel");
-      close (fd);
+      if (fd != 0)
+        {
+          close (fd);
+	}
       tunnel_close (tunnel);
       log_notice ("disconnected from FIXME:hostname:port");
     }
