@@ -36,8 +36,6 @@ USA.  */
 #include <config.h>
 #endif
 
-#ifndef HAVE_GETOPT_LONG
-
 #if !defined (__STDC__) || !__STDC__
 /* This is a separate conditional since some stdc systems
    reject `defined (const)'.  */
@@ -115,7 +113,7 @@ USA.  */
    GNU application programs can use a third alternative mode in which
    they can distinguish the relative order of options and other arguments.  */
 
-#include "getopt_.h"
+#include "getopt.h"
 
 /* For communication from `getopt' to the caller.
    When `getopt' finds an option that takes an argument,
@@ -1058,5 +1056,3 @@ main (argc, argv)
 }
 
 #endif /* TEST */
-
-#endif /* !HAVE_GETOPT_LONG */
