@@ -14,11 +14,14 @@ Tunnel *tunnel_new_client (const char *host, int host_port,
 
   Create a new HTTP tunnel client.
 
-Tunnel *tunnel_new_server (int port,
+Tunnel *tunnel_new_server (const char *host,
+                           int port,
 			   size_t content_length);
 
   Create a new HTTP tunnel server.  If LENGTH is 0, the Content-Length
   of the HTTP GET response will be determined automatically in some way.
+  If HOST is not NULL, use it to bind the server socket to a specific
+  network interface.
 
 int tunnel_connect (Tunnel *tunnel);
 
@@ -67,7 +70,8 @@ typedef struct tunnel Tunnel;
 extern Tunnel *tunnel_new_client (const char *host, int host_port,
 				  const char *proxy, int proxy_port,
 				  size_t content_length);
-extern Tunnel *tunnel_new_server (int port, size_t content_length);
+extern Tunnel *tunnel_new_server (const char *host, int port,
+                                  size_t content_length);
 extern int tunnel_connect (Tunnel *tunnel);
 extern int tunnel_accept (Tunnel *tunnel);
 extern int tunnel_pollin_fd (Tunnel *tunnel);
